#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import subprocess
import re
import time
from multiprocessing import Pool, Manager
#作業ディレクトリ用
import os
import shutil
#
import wave
# デバッグフラグ
DEBUG = True

# 音声合成の同時処理数
MAX_PROCESS = 3

# aplayコマンドの場所
APLAY = '/usr/bin/aplay'

# 作業ディレクトリ
WORKDIR = "/home/pi/openjtalk/tmp/"

#
# デバッグメッセージ出力用
#
def _print( message ):
        if DEBUG:
                print (message)

#
def join_waves():
    inputs = os.listdir('/home/pi/openjtalk/tmp')
    inputs.sort()
    output = WORKDIR + "total.wav"
    try:
        fps = [wave.open(WORKDIR + f, 'r') for f in inputs]
        fpw = wave.open(output, 'w')
        
        fpw.setnchannels(fps[0].getnchannels())
        fpw.setsampwidth(fps[0].getsampwidth())
        fpw.setframerate(fps[0].getframerate())
        
        for fp in fps:
            fpw.writeframes(fp.readframes(fp.getnframes()))
            fp.close()
        fpw.close()
        
        #
                                
        return "ok"
    
    except:
        print("join_waves Error")
        return "no"
#
# 音声合成を実行する
#
def create_wav(t):
        _print( "DEBUG: 音声作成開始[%d:%s]" % ( t[0], t[1] ) )
        # サブプロセス呼び出し
        
        outfile = WORKDIR + "talk%02d.wav" % t[0]
        
        #open_jtalk = ['open_jtalk']
        #open_jtalk = ['/usr/local/bin/open_jtalk']
        open_jtalk = ['/home/pi/open_jtalk-1.11/bin/open_jtalk']
        
        dict = ['-x','/var/lib/mecab/dic/open-jtalk/naist-jdic']
        
        #htsvoice = ['-m','/home/pi/openjtalk/mei_happy.htsvoice']
        #htsvoice = ['-m','/home/pi/openjtalk/mei_bashful.htsvoice']
        htsvoice = ['-m','/home/pi/openjtalk/mei_normal.htsvoice']
        
        halftone = ['-fm','0.55']
        allpass = ['-a','0.5']
        gvweight4fo = ['-jf','3.0']
        speed = ['-r','1.0']
        volume = ['-g','15.0']
        
        outwav = ['-ow',outfile]
        cmd = open_jtalk + dict + htsvoice + speed + halftone + allpass+ volume + gvweight4fo + outwav
        p = subprocess.Popen(cmd,stdin=subprocess.PIPE)
        p.stdin.write(t[1].encode('utf-8'))
        p.stdin.close()
        p.wait()
        
        _print( "DEBUG: 音声作成終了[%d:%s]" % ( t[0], t[1] ) )
        t[2].put(t[0])
        return c.returncode

#
# 音声を(複数まとめて)再生する
#
def play_wav(listindex):
        command = list()
        command.append(APLAY)
        #command.append('-q')
        command.append('-Dplughw:1,0')
        for index in listindex:
                command.append(WORKDIR + "talk%02d.wav" % index)
        return subprocess.Popen(command)

def jtalk(words):
    
    if(os.path.isdir(WORKDIR)):
        shutil.rmtree(WORKDIR)
        os.mkdir(WORKDIR)
    
    queue = Manager().Queue()
    index = 0
    arg = list()
    #shorttext = re.split(r'、|。', words)
    shorttext = re.split(r'、|。-[3,6]', words)
    for s in shorttext:
        if s != "":
            arg.append((index, s, queue))
            index += 1
    
    _print( "DEBUG: %d個に分割しました。" % len(arg) )
    
    # プロセスプールを作成
    p = Pool(MAX_PROCESS)
    
    r = p.map_async( create_wav, arg )
    
    l = [ 0 for i in range(len(arg))]
    
    l.append(-1)
    
    nowplaying = -1
    
    playing = None
    
    #
    
    while ( not r.ready() ) or ( nowplaying != len(arg) ) or ( playing is not None ):
        time.sleep(0.5)
        # 音声合成の終了報告があるかキューを確認する。
        for _ in range(queue.qsize()):
                compiled_index = queue.get()
                l[compiled_index] = 1
        # 再生できるならしてみる
        if nowplaying < len(arg):
                if playing is None:
                        if l[nowplaying + 1] == 1:
                                # まとめてWAVファイルを指定できるときはする
                                listindex = list()
                                while l[nowplaying + 1] == 1:
                                        nowplaying += 1
                                        listindex.append(nowplaying)
                                
                                _print( "DEBUG: しゃべるよ！[%s]" % str(listindex) )
                                playing = play_wav(listindex)
                        elif l[nowplaying + 1] == 0:
                                _print( "DEBUG: 音声合成の完了待ちです！" )
                        else:
                                print("音声再生終了" + str(len(listindex)))
                                
                                ret = join_waves()
                                exit()
                else:
                        if playing.poll() is not None:
                                playing = None

